import datetime, sys
import getopt
from MaKaC.conference import CategoryManager
from indico.core.db import DBMgr
from indico.web.flask.app import make_app

def helpMsg():
    print 'impexpTemplates.py [-c] [-b] [-i <inputfile>]'
    print 'without arguments it will save the templates otherwise will import the indicated mandatory file'
    print '-c will clear the current set of templates before starting the operation'
    print '-b will work on badges otherwise on emails'


def main(argv):
    try:
      opts, args = getopt.getopt(argv,"hi:cb",["ifile="])
    except getopt.GetoptError:
        helpMsg()
        sys.exit(2)

    isImport=False
    clearBefore=False
    workOnBadges=False
    for opt, arg in opts:
        if opt == '-h':
            helpMsg()
            sys.exit()
        elif opt in ("-i", "--ifile"):
            inputfile = arg
            isImport=True
        elif opt == '-c':
            clearBefore=True
        elif opt == '-b':
            workOnBadges=True

    with make_app().app_context():
        dbi = DBMgr.getInstance()
        dbi.startRequest()
        conf = CategoryManager().getDefaultConference()
        mgr = conf.getBadgeTemplateManager() if workOnBadges else conf.getEmailTemplateManager()
        prefix = 'badgeTemplates' if workOnBadges else 'emailTemplates'
        if clearBefore:
            for k in mgr.getTemplates().keys():
                mgr.deleteTemplate(k)

        if isImport:
            mgr.importItems(inputfile)
        else:
            timestamp = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            mgr.exportItems('{} {}'.format(prefix,timestamp))
        dbi.endRequest()

if __name__ == "__main__":
   main(sys.argv[1:])