#!/bin/sh
#title           :restore.sh
#description     :This script will restore a backup created with backup.sh in INSTANCE_HOME/backup dir.
#author		     :Giorgio Pieretti
#date            :2015-11-20
#version         :0.1
#==============================================================================

SCRIPT_DIR=$(dirname $0)
INDICO_HOME=$SCRIPT_DIR/../..

INSTANCE_HOME=$(grep LogDir $INDICO_HOME/etc/indico.conf | cut -d '=' -f 2 | cut -d '"' -f 2)/..

###############################
# DO NOT CHANGE BELOW THIS LINE

BACKUP_HOME=$INSTANCE_HOME/backup
DB_NAME=indico
CONF=0
NOFIXLINKS=0



function execute_backup {
    # First of all, execute a backup:
    echo "****** Backing up..."
    $INDICO_HOME/bin/unog/backup.sh
}


function restore_psql {
    DBEXIST=$(psql -lqt | cut -d \| -f 1 | grep -w " $DB_NAME " | wc -l)
    if [[ $DBEXIST -eq 1 ]]
    then
        echo "****** Dropping existing Database"
        dropdb $DB_NAME
    fi
    createdb -T template0 $DB_NAME > /dev/null
    psql $DB_NAME < $BACKUP_DIR/pg.db > /dev/null
    echo "****** PostgreSQL restored"
}


function restore_zodb {
    mkdir -p $INSTANCE_HOME/db
    zdaemon -C $SCRIPT_DIR/../../etc/zdctl.conf stop
    repozo -Rv -r $BACKUP_DIR -o $INSTANCE_HOME/db/Data.fs > /dev/null
    zdaemon -C $SCRIPT_DIR/../../etc/zdctl.conf start
    echo "****** ZODB restored"
}

function restore_dirs {
    cp -R $BACKUP_DIR/archive $INSTANCE_HOME > /dev/null
    if [ "$CONF" == "1" ]; then
        cp -R $BACKUP_DIR/etc $INDICO_HOME > /dev/null
    fi
    if [ -d "$BACKUP_DIR/crypto" ]; then
        cp -R $BACKUP_DIR/crypto $INSTANCE_HOME > /dev/null
    fi
    if [ -d "$BACKUP_DIR/passes" ]; then
        cp -R $BACKUP_DIR/passes $INSTANCE_HOME > /dev/null
    fi
    echo "****** Dirs restored"
}

function fix_external_links {
    if [ "$NOFIXLINKS" == "0" ]; then
        python $SCRIPT_DIR/unog_manage_links.py -r $(cat $BACKUP_DIR/baseUrl.txt) > /dev/null
        echo "****** External Links FIXED"
    fi
}

function restoreit {
    echo "Going for the Restore. All existing data will be OVERWRITTEN. Go ahead? [y/n]:"
    read CONFIRM
    if [ "$CONFIRM" == "y" ]; then
        echo "ARE YOU REALLY SURE? [y/n]"
        read CONFIRM
        if [ "$CONFIRM" == "y" ]; then
            echo "****** Trying to STOP APACHE"
            service httpd stop
            restore_psql
            restore_zodb
            fix_external_links
            restore_dirs
            echo "****** Trying to RESTART APACHE"
            service httpd start
            echo "****** Restore Done."
        fi

    fi
}

function checkbackupdir {
    if [ ! -f "$BACKUP_DIR/pg.db" ]; then
        echo "ERROR: NO PostgreSQL backup file found!"
        return 1
    fi
    if [ ! -d "$BACKUP_DIR/archive" ]; then
        echo "ERROR: NO Archive backup dir found!"
        return 1
    fi
    countdat=`ls -1 $BACKUP_DIR/*.dat 2>/dev/null | wc -l`
    countfs=`ls -1 $BACKUP_DIR/*.fs 2>/dev/null | wc -l`
    countindex=`ls -1 $BACKUP_DIR/*.index 2>/dev/null | wc -l`
    if [ $countdat == 0 ] || [ $countfs == 0 ] || [ $countindex == 0 ]; then
        echo "ERROR: Some Repozo backup files are MISSING!"
        return 1
    fi
    if [ ! -d "$BACKUP_DIR/etc" ]; then
        echo "WARNING: NO etc dir found. You will not be able to restore configuration files."
        return 0
    fi
    return 0
}



function usage {
    echo "**************************************************************************************************"
    echo "* usage: restore [ [[-p zipfile] | [-r dirname [-etc] [-nofixlinks]]] | [-h] ]                               *"
    echo "*                                                                                                *"
    echo "* -p zipfile: PREPARE -> Prapare the ZIP file: extract and check                                 *"
    echo "* -r dirname: RESTORE -> Use dirname in backup dir (INDICO_INSTANCE/backup) as a restore source  *"
    echo "*                                                                                                *"
    echo "**************************************************************************************************"
}


for var in "$@"
do
    if [ "$var" == "-etc" ]; then
        CONF=1
    fi
    if [ "$var" == "-nofixlinks" ]; then
        NOFIXLINKS=1
    fi
done


if [ "$1" != "" ]; then

    case $1 in

        -p ) # Prepare
                                if [ "$2" != "" ]; then
                                    BACKUP_ZIP_NAME=$2
                                    BACKUP_DIR_NAME=${BACKUP_ZIP_NAME%.*}
                                    BACKUP_DIR=$BACKUP_HOME/$BACKUP_DIR_NAME
                                    if [ "$BACKUP_ZIP_NAME" == "" ] || [ ! -f "$BACKUP_HOME/$BACKUP_ZIP_NAME" ]; then
                                        echo "NO BACKUP file found."
                                        exit 1
                                    fi

                                    echo "***** Using file: " $BACKUP_ZIP_NAME
                                    echo "***** Continue with the extraction? [y/n]"
                                    read CONFIRM
                                    if [ "$CONFIRM" == "y" ]; then
                                        if [ -d "$BACKUP_DIR" ]; then
                                            echo "Directory already exist. Do you want to overwrite? [y/n]"
                                            read CONFIRM
                                            if [ "$CONFIRM" != "y" ]; then
                                                exit 1
                                            fi
                                        fi

                                        echo "****** Extracting zip file"
                                        unzip -o $BACKUP_HOME/$BACKUP_ZIP_NAME -d $BACKUP_HOME > /dev/null
                                        echo "****** Checking extracted files"

                                        checkbackupdir
                                        res=$?
                                        if [ $res == 1 ]; then
                                            echo "EXTRACTED ZIP does not contain enough data for a complete restore!"
                                        else
                                            echo "Completed."
                                        fi

                                        exit 0
                                    fi

                                fi
                                ;;
        -r ) # Restore
                                if [ "$2" != "" ]; then

                                    BACKUP_DIR_NAME=$2
                                    BACKUP_DIR=$BACKUP_HOME/$BACKUP_DIR_NAME
                                    if [ ! -d "$BACKUP_DIR" ]; then
                                        echo "Specified backup dir NOT FOUND."
                                        exit 1
                                    fi
                                    echo "***** Using backup dir: " $BACKUP_DIR_NAME
                                    checkbackupdir
                                    res=$?
                                    if [ $res == 1 ]; then
                                        echo "EXTRACTED ZIP does not contain enough data for a complete restore!"
                                        exit 1
                                    fi
                                    echo "Continue with Restore? [y/n]:"
                                    read CONFIRM
                                    if [ "$CONFIRM" == "y" ]; then
                                        echo "What about a BACKUP first... Do you want me to do it? [y/n]"
                                        read CONFIRM
                                        if [ "$CONFIRM" == "y" ]; then
                                            execute_backup
                                        else
                                            echo "So you feel lucky today... But I want to give you a second opportunity: do you want me to do a BACKUP now? [y/n]"
                                            read CONFIRM
                                            if [ "$CONFIRM" == "y" ]; then
                                                echo "Good choice!"
                                                execute_backup
                                            else
                                                echo "OK, it's your fault...!"
                                            fi
                                        fi
                                    restoreit
                                    fi
                                    exit 0

                                fi
                                ;;

        -h ) # Help
                                usage
                                exit
                                ;;
        * )                     usage
                                exit 1
    esac
else
    usage
    exit 1
fi
