# -*- coding: utf-8 -*-

try: from indico.core import db
except: from MaKaC.common import db
from indico.web.flask.app import make_app
from MaKaC.user import Avatar, AvatarHolder, Group, GroupHolder, LoginInfo
from MaKaC.authentication import AuthenticatorMgr
from indico.MaKaC.common import info
import sys, os, random, string

groups = [ {'name':'conference_officers','description':'', 'email':'', 'id':'0'},
               {'name':'security_officers','description':'', 'email':'', 'id':'1'},
               {'name':'super_admin','description':'', 'email':'', 'id':'2'}
            ]

users = [
    {'name':'Admin',
    'surname': 'Admin',
    'org': 'Unog',
    'lang': 'en_GB',
    'email': 'noreply.accreditation@unog.ch',
    'login':'admin',
     'groups': ['conference_officers','security_officers','super_admin'],
    'isadmin': True },

    {'name':'Remy',
    'surname': 'Vanderplaetsen',
    'org': 'Unog',
    'lang': 'en_GB',
    'email': 'rvanderplaetsen@unog.ch',
    'login':'vanderplaetsen',
    'groups': ['security_officers'],
    'isadmin': False},

    {'name':'David',
    'surname': 'Louvet',
    'org': 'Unog',
    'lang': 'en_GB',
    'email': 'dlouvet@unog.ch',
    'login':'louvet',
    'groups': ['security_officers'],
    'isadmin': False},

    {'name':'Giorgio',
    'surname': 'Pieretti',
    'org': 'Unog',
    'lang': 'en_GB',
    'email': 'gpieretti@unog.ch',
    'login':'pieretti',
    'groups': ['conference_officers','security_officers','super_admin'],
    'isadmin': True},

    {'name':'Claudio',
    'surname': 'Morgia',
    'org': 'Unog',
    'lang': 'en_GB',
    'email': 'cmorgia@unog.ch',
    'login':'morgia',
    'groups': ['conference_officers','security_officers','super_admin'],
    'isadmin': True},

    {'name':'Da',
    'surname': 'Nzete',
    'org': 'Unog',
    'lang': 'en_GB',
    'email': 'ndasamaitoua@unog.ch',
    'login':'nzete',
    'groups': ['conference_officers','security_officers','super_admin'],
    'isadmin': True},

    {'name':'Vu',
    'surname': 'Tran',
    'org': 'Unog',
    'lang': 'en_GB',
    'email': 'vtran@unog.ch',
    'login':'tran',
    'groups': ['conference_officers','security_officers','super_admin'],
    'isadmin': True},

]

class UnogInitUsers():
    def __init__(self):
        self.gh = GroupHolder()
        self.ah = AvatarHolder()
        self._authMgr = AuthenticatorMgr()

    def createGroups(self):
        try:
            for g in groups:
                print "Managing group: " + g['name']
                if g['name'] not in [g.getName() for g in GroupHolder().getValuesToList()]:
                    group = Group()
                    group.setName(g['name'])
                    group.setDescription(g['description'])
                    self.gh.add(group)
                    print "Group creation... Done."
                else:
                    print "Group name ALREADY EXIST."
        except Exception as e:
            print "Groups creation... FAILED: " + str(e)

        return

    def createRandomPassword(self):
        length = 13
        chars = string.ascii_letters + string.digits + '!@#$%^&*()'
        random.seed = (os.urandom(1024))
        return ''.join(random.choice(chars) for i in range(length))

    def createUsers(self):
        for user in users:
            try:
                print "Creating user: " + user['name'] + " " + user['surname']
                avatar = Avatar()
                avatar.setName(user['name'].encode('utf-8'))
                avatar.setSurName(user['surname'])
                avatar.setOrganisation(user['org'])
                avatar.setLang(user['lang'])
                avatar.setEmail(user['email'])
                avatar.activateAccount()
                for group in self.gh.getValuesToList():
                    if group.getName() in user['groups']:
                        print "Adding to group " + group.getName()
                        group.addMember(avatar)
                identity = self._authMgr.createIdentity(LoginInfo(user['login'], self.createRandomPassword()), avatar, "Local")
                self._authMgr.add(identity)
                self.ah.add(avatar)
                if user['isadmin']:
                    # add to admin list
                    print "Adding to AdminList"
                    minfo = info.HelperMaKaCInfo.getMaKaCInfoInstance()
                    adminList = minfo.getAdminList()
                    adminList.grant(avatar)
                print "User creation... Done."
            except Exception as e:
                print "User creation... FAILED: " + str(e)
        return








if __name__ == '__main__':

    if len(sys.argv) == 1 or '--h' in sys.argv or '-h' in sys.argv:
        print " ***********   UNOG default Users and Group creator    **********"
        print " *                                                              *"
        print " * Available options:                                           *"
        print " * -doit: creates groups and default users.                     *"
        print " *  Passwords are randomly generated. Use the LOST PASS feature *"
        print " *  to retrieve them.                                           *"
        print " *                                                              *"
        print " ****************************************************************"

    if '-doit' in sys.argv:
        with make_app().app_context():
            dbi = db.DBMgr.getInstance()
            dbi.startRequest()
            uiu = UnogInitUsers()
            doit = uiu.createGroups()
            doit = uiu.createUsers()
            dbi.endRequest()