# -*- coding: utf-8 -*-
import sys
from MaKaC.common.indexes import IndexesHolder
from MaKaC.user import AvatarHolder, GroupHolder
from indico.MaKaC.common import info
from indico.core.db import DBMgr
from MaKaC.conference import ConferenceHolder, CategoryManager
from indico.web.flask.app import make_app
from MaKaC.common import indexes
from MaKaC.authentication.AuthenticationMgr import AuthenticatorMgr


ch = ConferenceHolder()
cm = CategoryManager()

def cleanIndexes():

    ah = AvatarHolder()

    # Remove non existing avatar from the indexes
    for i in ah._indexes:
        print "Checking Index " + i
        d = IndexesHolder().getById(i)._words
        new_d = {}
        for key in d.keys():
            newList = []
            for avatarId in d[key]:
                avatar = AvatarHolder().getById(avatarId)
                if avatar:
                    newList.append(avatarId)
                else:
                    print "Removing Id " + avatarId + " from " + key
            new_d[key] = newList
        IndexesHolder().getById(i).setIndex(new_d)



    # Remove zombies from indexes:
    for i in ah._indexes:
        d = IndexesHolder().getById(i)._words
        new_d = dict((k, v) for k, v in d.iteritems() if v)
        IndexesHolder().getById(i).setIndex(new_d)
        print "Cleaning index", i


def kill( id = None ):
    if not id: return None
    ah = AvatarHolder()
    am = AuthenticatorMgr()
    avatar = ah.getById(id)

    if avatar:
        print "Managing Avatar", avatar
        # Remove all Conference Registrations:
        for reg in avatar.getRegistrantList():
            # Remove the Registrant from the Conference
            conf = reg.getConference()
            for r in conf.getRegistrantsList():
                a = r.getAvatar()
                if a and a.getId() == avatar.getId():
                    conf.removeRegistrant(r.getId())
            avatar.removeRegistrant(reg)
            print "Removing Registrant from Conference ",reg.getConference().getTitle()


        # Remove Identities from Avatar and Authenticator

        for identity in avatar.getIdentityList():
            print "Removing Identity ", identity
            avatar.removeIdentity(identity.getId())
            auth = am.getById(identity.getAuthenticatorTag())
            auth.remove(identity)


        # Remove Avatar from indexes
        for i in ah._indexes:
            # This creates zombies!
            indexes.IndexesHolder().getById(i).unindexUser(avatar)
            print "Removing Avatar from Index: ",i

        # Remove from Groups
        gh = GroupHolder()
        for group in gh.getValuesToList():
            if avatar.is_member_of_group(group.getName()):
                print "Removing from Group:", group.getName()
                group.removeMember(avatar)
                dbi.commit()


        # Remove Avatar from Adminlist
        minfo = info.HelperMaKaCInfo.getMaKaCInfoInstance()
        adminList = minfo.getAdminList()
        adminList.revoke(avatar)


        # Remove Avatar from Category (in case of granted Admin access)
        for cat in cm.getValuesToList():
            if avatar in cat.getAccessController().getModifierList():
                cat.getAccessController().revokeModification(avatar)
                print "Removed avatar from Admin list in Category", cat

        # Remove Avatar from Holder:
        ah.remove(avatar)
        print "Removed Avatar from Holder"


        avatar.delete()
        print "Deleted Avatar"




    dbi.commit()
    return True



def kill_conf_registrants( id = None ):
    if not id: return None
    conf = ch.getById(id)
    for registrant in conf.getRegistrantsList():
        print "Managing " + registrant.getFullName()
        avatar = registrant.getAvatar()

        if avatar and len(avatar.getRegistrantList()) == 1:
            print "Removing AVATAR"
            kill(avatar.getId()) # by removing the Avatar, I also remove the related Registrant
        else:
            print "Avatar NOT removed: more Registrants existing"
            print "Removing ONLY the Registrant"
            conf.removeRegistrant(registrant.getId())




if __name__ == '__main__':

    if len(sys.argv) == 1 or '--h' in sys.argv or '-h' in sys.argv:
        print " *****************     UNOG Indico Avatar KILLER    *******************"
        print " *                                                                    *"
        print " * Available options:                                                 *"
        print " * -id #: remove Avatar with specified ID. Can be more                *"
        print " *         than one: eg.: -id 5 7 18                                  *"
        print " * -all: remove ALL Avatars EXCEPT Administrators OR part of a group. *"
        print " *                                                                    *"
        print " * -confid #: remove all Registrants and Avatar (if no other          *"
        print " *         Registrants found) for that Conference                     *"
        print " *                                                                    *"
        print " * -fix: only fix the indexes                                         *"
        print " **********************************************************************"

    if '-id' in sys.argv:
        pos = sys.argv.index('-id')
        ids = sys.argv[pos+1:]
        with make_app().app_context():
            dbi = DBMgr.getInstance()
            dbi.startRequest()
            for id in ids:
                kill(int(id))
            cleanIndexes()
            dbi.endRequest()

    if '-confid' in sys.argv:
        pos = sys.argv.index('-confid')
        ids = sys.argv[pos+1:]
        with make_app().app_context():
            dbi = DBMgr.getInstance()
            dbi.startRequest()
            for id in ids:
                kill_conf_registrants(int(id))
            cleanIndexes()
            dbi.endRequest()

    if '-all' in sys.argv:
        sys.stdout.write("Are REALLY sure? [y/n]: ")
        choice = raw_input().lower()
        if choice == 'y':
            with make_app().app_context():
                dbi = DBMgr.getInstance()
                dbi.startRequest()
                ah = AvatarHolder()
                gh = GroupHolder()
                glist = gh.getValuesToList()
                for avatar in ah.getValuesToList():
                    if not avatar.isAdmin():
                        # check if not part of any group
                        remove = True
                        for group in glist:
                            if avatar.is_member_of_group(group.getName()):
                                remove = False
                        if remove: kill(int(avatar.getId()))
                cleanIndexes()
                dbi.endRequest()
        else:
            print "No action taken."

    if '-fix' in sys.argv:
        with make_app().app_context():
            dbi = DBMgr.getInstance()
            dbi.startRequest()
            cleanIndexes()
            dbi.endRequest()


