# This file is part of Indico.
# Copyright (C) 2002 - 2015 European Organization for Nuclear Research (CERN).
#
# Indico is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# Indico is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Indico; if not, see <http://www.gnu.org/licenses/>.
import datetime

from MaKaC.badgeDesignConf import prettyDateRange
from MaKaC.webinterface.common.countries import CountryHolder

from indico.core.config import Config

from MaKaC.services.implementation.base import ServiceBase
from user import AvatarHolder
import pytz

class Redirect(ServiceBase):

    def _checkParams(self):
        ServiceBase._checkParams(self)
        self.firstName = self._params['firstName'] if 'firstName' in self._params else None
        self.lastName = self._params['lastName'] if 'lastName' in self._params else None
        self.birthDate = self._params['birthDate'] if 'birthDate' in self._params else None
        self.passportID = self._params['passportID'] if 'passportID' in self._params else None
        self.passportOrigin = self._params['passportOrigin'] if 'passportOrigin' in self._params else None
        self.passportOrigin = CountryHolder().getCountry32(self.passportOrigin)

    def _getAnswer(self):
        avs = AvatarHolder().match({'passport': self.passportOrigin+self.passportID})
        if len(avs)<1:
            return {"error":"Avatar not found"}

        av = avs[0]
        nowDate = pytz.utc.localize(datetime.datetime.today())
        response = []
        for reg in av.getRegistrantList():
            if reg.getConference().getEndDate() >= nowDate:
                item = dict()
                item['description']="{} - {}".format(reg.getConference().getTitle(),prettyDateRange(reg.getConference().getStartDate(),reg.getConference().getEndDate()))
                item['current']=reg.getConference().isCurrent()
                item['link']= '{}/event/{}/registration/modify?registrantId={}&embedded=True'.format(
                    Config.getInstance().getBaseURL(),
                    reg.getConference().getId(),
                    reg.getId()
                )
                response.append(item)

        return response


methodMap = {
    "redirect": Redirect
}
