# This file is part of Indico.
# Copyright (C) 2002 - 2015 European Organization for Nuclear Research (CERN).
#
# Indico is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# Indico is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Indico; if not, see <http://www.gnu.org/licenses/>.

from xml.sax.saxutils import quoteattr, escape
from MaKaC.i18n import _
from reportlab.lib import colors

class ContactTypesRegistry:

    # _items = ["Head of Delegation", "Delegate", "Press", "UN Staff member", "Intern", "Consultant", "Other"]
    _items = ["Head of Delegation", "Delegate", "Observer Organization", "Press", "UN Staff member", "NGO", "Guest"]

    _mapTemplates = {
        "Head of Delegation": 'template_default',
        "Delegate": 'template_default',
        "Observer Organization": 'template_default',
        "Press": 'template_default',
        "UN Staff member": 'template_default',
        "NGO": 'template_default',
        "Guest": 'template_default'
    }

    _mapColors =  {
        "Head of Delegation": [colors.yellow, 'yellow','D'],
        "Delegate": [colors.green,'green', 'D'],
        "Observer Organization": [colors.green,'green', 'O'],
        "UN Staff member": [colors.blue,'blue','S'],
        "Press": [colors.red,'red','P'],
        "NGO": [colors.red,'red','N'],
        "Guest": [colors.red,'red','Z'],
        "default": [colors.transparent,'transparent','']
            }

    @classmethod
    def getList( cls ):
        return cls._items

    @classmethod
    def getSelectItemsHTML( cls, selContact="" ):
        l=[]
        for contact in cls._items:
            selected=""
            if contact==selContact:
                selected=" selected"
            if contact != "":
                contact=_(contact)
            l.append("""<option value=%s%s>%s</option>"""%(quoteattr(contact),
                                        selected, escape(contact)))
        return "".join(l)

    @classmethod
    def getMapBadgeTemplates(cls):
        return cls._mapTemplates

    @classmethod
    def getMapColor(cls, contactType):
        map = cls._mapColors
        if contactType in map:
            return map[contactType][0]
        return map['default'][0]

    @classmethod
    def getMapColorName(cls, contactType):
        map = cls._mapColors
        if contactType in map:
            return map[contactType][1]
        return map['default'][1]

    @classmethod
    def getMapLetter(cls, contactType):
        map = cls._mapColors
        if contactType in map:
            return map[contactType][2]
        return map['default'][2]
