# This file is part of Indico.
# Copyright (C) 2002 - 2015 European Organization for Nuclear Research (CERN).
#
# Indico is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# Indico is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Indico; if not, see <http://www.gnu.org/licenses/>.
from MaKaC.user import Avatar
from indico.web.flask.util import url_for
import MaKaC.webinterface.urlHandlers as urlHandlers
import MaKaC.review as review
from MaKaC.common import log
from MaKaC.webinterface.mail import GenericMailer
from MaKaC.webinterface.common.baseNotificator import TplVar, Notification
import base64
from qrcode import QRCode, constants
from indico.util import json
from MaKaC.common.Configuration import Config
from indico.util.date_time import format_date
from cStringIO import StringIO
from indico.modules.oauth.db import ConsumerHolder
from urlparse import urlparse,urlunparse

class ConfTitleTplVar(TplVar):
    _name="conference_title"
    _description=""

    def getValue(cls,registrant):
        return registrant.getConference().getTitle()
    getValue=classmethod(getValue)


class ConfURLTplVar(TplVar):
    _name="conference_URL"
    _description=""

    def getValue(cls,registrant):
        from indico.util.contextManager import ContextManager
        conf = ContextManager.get('currentConf', None)
        if not conf:
            conf = registrant.getConference()
        return str(urlHandlers.UHConferenceDisplay.getURL(conf))
    getValue=classmethod(getValue)


class ConfLogoTplVar(TplVar):
    _name="conference_logo"
    _description=""

    def getValue(cls,registrant):
        from indico.util.contextManager import ContextManager
        conf = ContextManager.get('currentConf', None)
        if not conf:
            conf = registrant.getConference()
        url = str(urlHandlers.UHConferenceDisplay.getURL(conf))+"logo"
        result = '<img src="{}" style="width:500px" />'.format(url)
        return result
    getValue=classmethod(getValue)


class ConfSupportEmailTplVar(TplVar):
    _name="conference_support_email"
    _description=""

    def getValue(cls,registrant):
        from indico.util.contextManager import ContextManager
        conf = ContextManager.get('currentConf', None)
        if not conf:
            conf = registrant.getConference()
        return '<a href="mailto:{0}">{0}</a>'.format(conf.getSupportInfo().getEmail())
    getValue=classmethod(getValue)


class RegistrantIdTplVar(TplVar):
    _name="registrant_id"
    _description=""

    def getValue(cls,registrant):
        return registrant.getId()
    getValue=classmethod(getValue)


class RegistrantFirstNameTplVar(TplVar):
    _name="registrant_first_name"
    _description=""

    def getValue(cls,registrant):
        return registrant.getFirstName()
    getValue=classmethod(getValue)

class RegistrantFamilyNameTplVar(TplVar):
    _name="registrant_family_name"
    _description=""

    def getValue(cls,registrant):
        return registrant.getFamilyName()
    getValue=classmethod(getValue)

class RegistrantTitleTplVar(TplVar):
    _name="registrant_title"
    _description=""

    def getValue(cls,registrant):
        return registrant.getTitle()
    getValue=classmethod(getValue)


class RegistrantMeetingTplVar(TplVar):
    _name="registrant_meeting"
    _description=""

    def getValue(cls,registrant):
        sessions = registrant.getSessionList() or registrant.getAllSessions()
        sessions = [s.getSession() for s in sessions]
        import datetime, pytz
        now = datetime.datetime.utcnow()
        now = now.replace(tzinfo=pytz.utc)
        sessions = filter(lambda s: s.getStartDate() <= now <= (s.getStartDate()+s.getDuration()),sessions)
        return sessions[0].getTitle() if len(sessions)>0 else "NO MEETING"

    getValue=classmethod(getValue)


class RegistrantRoomTplVar(TplVar):
    _name="registrant_room"
    _description=""

    def getValue(cls,registrant):
        sessions = registrant.getSessionList() or registrant.getAllSessions()
        sessions = [s.getSession() for s in sessions]
        import datetime, pytz
        now = datetime.datetime.utcnow()
        now = now.replace(tzinfo=pytz.utc)
        sessions = filter(lambda s: s.getStartDate() <= now <= (s.getStartDate()+s.getDuration()),sessions)
        return sessions[0].getRoom().getName() if len(sessions)>0 else "NO MEETING"
    getValue=classmethod(getValue)


class RegistrantAccommodationTplVar(TplVar):
    _name="registrant_accommodation"
    _description=""

    def getValue(cls,registrant):
        if registrant.getAccommodation() is not None and \
                registrant.getAccommodation().getAccommodationType() is not None:
                    return registrant.getAccommodation().getAccommodationType().getCaption()
        return ""
    getValue=classmethod(getValue)

class RegistrantSocialEventsTplVar(TplVar):
    _name="registrant_social_events"
    _description=""

    def getValue(cls,registrant):
        selist=[]
        for se in registrant.getSocialEvents():
            selist.append("%s [%s place(s)]"%(se.getCaption(), se.getNoPlaces()))
        return ", ".join(selist)
    getValue=classmethod(getValue)

class RegistrantSessionsTplVar(TplVar):
    _name="registrant_sessions"
    _description=""

    def getValue(cls,registrant):
        selist=[]
        for se in registrant.getSessionList():
            selist.append(se.getTitle())
        return ", ".join(selist)
    getValue=classmethod(getValue)

class RegistrantArrivalDateTplVar(TplVar):
    _name="registrant_arrival_date"
    _description=""

    def getValue(cls,registrant):
        arrivalDate=""
        if registrant.getAccommodation() is not None and registrant.getAccommodation().getArrivalDate() is not None:
            arrivalDate = registrant.getAccommodation().getArrivalDate().strftime("%d-%B-%Y")
        return arrivalDate
    getValue=classmethod(getValue)

class RegistrantDepartureDateTplVar(TplVar):
    _name="registrant_departure_date"
    _description=""

    @classmethod
    def getValue(cls,registrant):
        departureDate=""
        if registrant.getAccommodation() is not None and registrant.getAccommodation().getDepartureDate() is not None:
            departureDate = registrant.getAccommodation().getDepartureDate().strftime("%d-%B-%Y")
        return departureDate


class RegistrantLinkTplVar(TplVar):
    _name = "registrant_link"
    _description = ""

    @classmethod
    def getValue(cls, registrant):
        params = {}
        params = {'registrantId': registrant.getId(), 'authkey': registrant.getRandomId()}
        reg_page = url_for('event.confRegistrationFormDisplay', registrant.getConference(), _external=True,
                           _secure=True, **params)
        burl = urlparse(Config.getInstance().getBaseURL())
        regurl = urlparse(reg_page)
        return urlunparse([burl.scheme,burl.netloc,regurl.path,regurl.params,regurl.query,regurl.fragment])


class ResetPasswordTplVar(TplVar):
    _name = "reset-password"
    _description = ""

    @classmethod
    def getValue(cls, registrant):
        from MaKaC.user import Avatar
        if isinstance(registrant, Avatar):
            user = registrant
            event = None
        else:
            user = registrant.getAvatar()
            event = registrant.getConference()
        from MaKaC.common.cache import GenericCache
        import uuid
        token_storage = GenericCache('resetpass')
        endpoint = 'event.confLogin-resetPassword' if event else 'user.signIn-resetPassword'

        idList = user.getIdentityList()
        logins = []
        urls=[]
        for id in idList:
            if hasattr(id, 'setPassword'):
                tag = id.getAuthenticatorTag()
                login = id.getLogin()
                data = {'tag': tag, 'login': login}
                token = str(uuid.uuid4())
                while token_storage.get(token):
                    token = str(uuid.uuid4())
                token_storage.set(token, data, 6*3600)
                url = url_for(endpoint, event, token=token, _external=True, _secure=True)
                logins.append({
                    'tag': tag,
                    'login': login,
                    'link': url
                })
        if not logins:
            return ""
        else:
            for entry in logins:
                if 'link' in entry:
                    urls.append(entry['link'])
        url = urls[0] if len(urls)>0 else None
        return "<a href=\"{}\">{}</a>".format(url,url)


class TicketLinkTplVar(TplVar):
    _name = "ticket-link"
    _description = ""

    @classmethod
    def getValue(cls, registrant):
        return "<a href=\"{}\">Download e-Ticket</a>".format(url_for("event.e-ticket-pdf",
                registrant,
                authkey=registrant.getRandomId(),_external=True))


class UserDetailsLinkTplVar(TplVar):
    _name = "userdetails-link"
    _description = ""

    @classmethod
    def getValue(cls, registrant):
        user = registrant if isinstance(registrant,Avatar) else registrant.getAvatar()
        return "<a href=\"{}\">user details</a>".format(urlHandlers.UHUserDetails.getURL( user ))


class RegistrantUserNameTplVar(TplVar):
    _name = "registrant-username"
    _description = ""

    @classmethod
    def getValue(cls, registrant):
        user = registrant if isinstance(registrant,Avatar) else registrant.getAvatar()
        return user.getEmail()


class ActivationLinkTplVar(TplVar):
    _name = "user-activation-link"
    _description = ""

    @classmethod
    def getValue(cls, registrant):
        if isinstance(registrant,Avatar):
            user = registrant
            from indico.util.contextManager import ContextManager
            conf = ContextManager.get('currentConference', None)

        else:
            user = registrant.getAvatar()
            conf = registrant.getConference()

        if not conf:
            url = "{}?userId={}&key={}".format(urlHandlers.UHActiveAccount.getURL(),user.getId(),user.getKey())
        else:
            returnURL = url_for('event.confRegistrationFormDisplay-creation', conf, _external=True, _secure=True)
            url = "{}&key={}&returnURL={}".format(urlHandlers.UHConfActiveAccount.getURL(conference=conf,av=user),user.getKey(),returnURL)

        return "<a href=\"{}\">Activation link</a>".format(url)


class UserFullNameTplVar(TplVar):
    _name = "user_full_name"
    _description = ""

    @classmethod
    def getValue(cls, registrant):
        user = registrant if isinstance(registrant,Avatar) else registrant.getAvatar()
        return user.getFullName()


class QRCodeTplVar(TplVar):
    _name = "registrant_qrcode"
    _description = ""

    @classmethod
    def getValue(cls, registrant):
        consumers = dict((consumer.getName(), consumer) for consumer in ConsumerHolder().getList())

        if "indico-checkin" not in consumers:
            raise Exception(_("There is no indico-checkin consumer key for OAuth"))

        qr = QRCode(
            version=1,
            error_correction=constants.ERROR_CORRECT_L,
            box_size=4,
            border=1
        )

        oauth_checkin = consumers["indico-checkin"]
        config = Config.getInstance()
        baseURL = config.getBaseSecureURL() if config.getBaseSecureURL() else config.getBaseURL()
        qr_data = {"r": registrant.getId(),
                   "e": registrant.getConference().getId()
                   }
        json_qr_data = json.dumps(qr_data)
        qr.add_data(json_qr_data)
        qr.make(fit=True)
        qr_img = qr.make_image()

        output = StringIO()
        qr_img._img.save(output, format="png")
        im_data = output.getvalue()

        return '<img src="data:image/png;base64,{0}"/>'.format(base64.b64encode(im_data))


class Notificator:
    _vars = [ConfTitleTplVar, ConfURLTplVar, ConfLogoTplVar, ConfSupportEmailTplVar, RegistrantIdTplVar,
             RegistrantFirstNameTplVar, RegistrantFamilyNameTplVar,
             RegistrantTitleTplVar, RegistrantAccommodationTplVar, RegistrantSocialEventsTplVar,
             RegistrantSessionsTplVar, RegistrantArrivalDateTplVar, RegistrantDepartureDateTplVar,
             RegistrantLinkTplVar, QRCodeTplVar, ResetPasswordTplVar, TicketLinkTplVar, RegistrantUserNameTplVar,
             UserDetailsLinkTplVar, ActivationLinkTplVar, RegistrantMeetingTplVar, RegistrantRoomTplVar,
             UserFullNameTplVar]

    @classmethod
    def getVarList(cls):
        return cls._vars

    @classmethod
    def collectParams(cls, body):
        # look for all tags in the body
        import re
        tags = re.findall("(\{.*?\})",body)
        params = []
        for tag in tags:
            params.append(tag.replace('{','').replace('}',''))
        return params

    def _getVars(self, registrant, body=None):
        d = {}
        if(body):
            params = self.collectParams(body)
            vars = filter(lambda k: k.getName() in params, Notificator.getVarList())
        else:
            vars = self.getVarList()

        for v in vars:
            d[v.getName()] = v.getValue(registrant)

        # TO BE FIXED BETTER
        from unidecode import unidecode

        if 'registrant_family_name' in d:
            d['registrant_family_name'] = unidecode(d['registrant_family_name'].decode("utf-8"))

        if 'registrant_first_name' in d:
            d['registrant_first_name'] = unidecode(d['registrant_first_name'].decode("utf-8"))
        return d


class EmailNotificator(Notificator):

    def apply(self,registrant,params):
        v=self._getVars(registrant, params.get("body",""))
        subj=params.get("subject","").format(**v)
        b=params.get("body","").format(**v)
        fa=params.get("from","")
        tl=params.get("to",[])
        cc = params.get("cc",[])

        b = "<html><body>"+b+"</body></html>"
        return Notification(subject=subj,body=b,fromAddr=fa,toList=tl,ccList=cc)

    def notify(self,registrant,params):
        if params.has_key("conf"):
            GenericMailer.sendAndLog(self.apply(registrant,params),
                                     params["conf"],
                                     log.ModuleNames.REGISTRATION)
        else:
            GenericMailer.send(self.apply(registrant,params))

    def notifyAll(self,params):
        subj=params.get("subject","")
        b=params.get("body","")
        fa=params.get("from","")
        tl=params.get("to",[])
        cc = params.get("cc",[])
        notification =  Notification(subject=subj,body=b,fromAddr=fa,toList=tl,ccList=cc)
        if params.has_key("conf"):
            GenericMailer.sendAndLog(notification, params["conf"],
                                     log.ModuleNames.REGISTRATION)
        else:
            GenericMailer.send(notification)
