from MaKaC.ecosoclist import ECOSOCList
from MaKaC.unctadlist import UNCTAD_IGO_list, UNCTAD_NGO_GEN_list, UNCTAD_NGO_SPE_list, UNCTAD_merged_list, UNS_mapping, UNCTAD_IOP_list

class UNListMapping:

            # 'ECOSOC Member States': [],
            # 'OHCHR Member States': [],
            # 'UNECE Member States': [],

    reprTypesDefault = {
        'Government': {
            'subtype': None,
            'contact': {'Delegate':[], 'Head of Delegation': []},
            'typeahead': False
        },
        'Intergovernmental Organizations': {
            'subtype': None,
            'contact': {'Observer Organization': []},
            'typeahead': True
        },
        'International Organizations of Parliamentarians': {
            'subtype': None,
            'contact': {'Observer Organization': []},
            'typeahead': True
        },
        'United Nations': {
            'subtype': {
                'UNCTAD (Programmes and Funds)': [],
                'ITC (Programmes and Funds)': [],
                'UNDP (Programmes and Funds)': [],
                'UNCDF (Programmes and Funds)': [],
                'UNV (Programmes and Funds)': [],
                'UNEP (Programmes and Funds)': [],
                'UNFPA (Programmes and Funds)': [],
                'UN-HABITAT (Programmes and Funds)': [],
                'UNHCR (Programmes and Funds)': [],
                'UNICEF (Programmes and Funds)': [],
                'UNODC (Programmes and Funds)': [],
                'UNRWA (Programmes and Funds)': [],
                'UN-Women (Programmes and Funds)': [],
                'WFP (Programmes and Funds)': [],
                'UNFCCC (Programmes and Funds)': [],

                'UNICRI (Research and Training Institutes)': [],
                'UNIDIR (Research and Training Institutes)': [],
                'UNITAR (Research and Training Institutes)': [],
                'UNRISD (Research and Training Institutes)': [],
                'UNSSC (Research and Training Institutes)': [],
                'UNU (Research and Training Institutes)': [],

                'UNAIDS (Other Entities)': [],
                'UNISDR (Other Entities)': [],
                'UNOPS (Other Entities)': [],

                'CTBTO (Related Organizations)': [],
                'IAEA (Related Organizations)': [],
                'OPCW (Related Organizations)': [],
                'WTO (Related Organizations)': [],

                'ILO (Specialized Agencies)': [],
                'FAO (Specialized Agencies)': [],
                'UNESCO (Specialized Agencies)': [],
                'WHO (Specialized Agencies)': [],
                'World Bank Group (Specialized Agencies)': [],
                'IBRD (Specialized Agencies)': [],
                'IDA (Specialized Agencies)': [],
                'IFC (Specialized Agencies)': [],
                'MIGA (Specialized Agencies)': [],
                'ICSID (Specialized Agencies)': [],
                'IMF (Specialized Agencies)': [],
                'ICAO (Specialized Agencies)': [],
                'IMO (Specialized Agencies)': [],
                'ITU (Specialized Agencies)': [],
                'UPU (Specialized Agencies)': [],
                'WMO (Specialized Agencies)': [],
                'WIPO (Specialized Agencies)': [],
                'IFAD (Specialized Agencies)': [],
                'UNIDO (Specialized Agencies)': [],
                'UNWTO (Specialized Agencies)': [],

                'Main and other sessional committees (Subsidiary Bodies)': [],
                'Disarmament Commission (Subsidiary Bodies)': [],
                'Human Rights Council (Subsidiary Bodies)': [],
                'International Law Commission (Subsidiary Bodies)': [],
                'Standing committees and ad hoc bodies (Subsidiary Bodies)': [],
                'Counter-terrorism committees (Subsidiary Bodies)': [],
                'International Criminal Tribunal for Rwanda (Subsidiary Bodies)': [],
                'International Criminal Tribunal for the former Yugoslavia (Subsidiary Bodies)': [],
                'Military Staff Committee (Subsidiary Bodies)': [],
                'Peacekeeping operations and political missions (Subsidiary Bodies)': [],
                'Sanctions committees ad hoc (Subsidiary Bodies)': [],

                'UN Peacebuilding Commission (Advisory Subsidiary Body)': [],

                'Crime Prevention and Criminal Justice (Functional Commissions)': [],
                'Narcotic Drugs (Functional Commissions)': [],
                'Population and Development (Functional Commissions)': [],
                'Science and Technology for Development (Functional Commissions)': [],
                'Social Development (Functional Commissions)': [],
                'Statistics (Functional Commissions)': [],
                'Status of Women (Functional Commissions)': [],
                'Sustainable Development (Functional Commissions)': [],
                'United Nations Forum on Forests (Functional Commissions)': [],

                'ECA (Regional Commissions)': [],
                'ECE (Regional Commissions)': [],
                'ECLAC (Regional Commissions)': [],
                'ESCAP (Regional Commissions)': [],
                'ESCWA (Regional Commissions)': [],

                'Committee for Development Policy (Other Bodies)': [],
                'Committee of Experts on Public Administration (Other Bodies)': [],
                'Committee on Non-Governmental Organizations (Other Bodies)': [],
                'Permanent Forum on Indigenous Issues (Other Bodies)': [],
                'United Nations Group of Experts on Geographical Names (Other Bodies)': [],
                'Other sessional and standing committees and experts (Other Bodies)': [],

                'EOSG (Departments and Offices)': [],
                'DESA (Departments and Offices)': [],
                'DFS (Departments and Offices)': [],
                'DGACM (Departments and Offices)': [],
                'DM (Departments and Offices)': [],
                'DPA (Departments and Offices)': [],
                'DPI (Departments and Offices)': [],
                'DPKO (Departments and Offices)': [],
                'DSS (Departments and Offices)': [],
                'OCHA (Departments and Offices)': [],
                'OHCHR (Departments and Offices)': [],
                'OIOS (Departments and Offices)': [],
                'OLA (Departments and Offices)': [],
                'OSAA (Departments and Offices)': [],
                'SRSG/CAAC (Departments and Offices)': [],
                'UNODA (Departments and Offices)': [],
                'UNOG (Departments and Offices)': [],
                'UN-OHRLLS (Departments and Offices)': [],
                'UNON (Departments and Offices)': [],
                'UNOV (Departments and Offices)': []
            },
            'contact': {'UN Staff member':[]},
            'typeahead': False
        },
        'Non Governmental Organizations': {
            'subtype': None,
            'contact': {'NGO': []},
            'typeahead': True
        },
        'Private Sector': {'subtype':None, 'contact':{'Guest':[]}, 'typeahead':False},
        'Academia': {'subtype':None, 'contact':{'Guest':[]}, 'typeahead':False},
        'Press/Media': {'subtype':None, 'contact':{'Press':[]}, 'typeahead':False},
        'Other': {'subtype':None, 'contact':{'Guest':[]}, 'typeahead':False}
    }

        # 'Member of the Conference (Holy See)'
        # 'Non-member observer State (State of Palestine)'


    reprTypesMapping = {
        #'clientid1': reprTypesDefault,
        '60.0010': reprTypesDefault,
        'default': reprTypesDefault
    }

    orgListsMapping = {
        # 'clientid1': orgList1,
        # '60.0010': {'IGO accredited': UNCTAD_IGO_list, 'IGO non-accredited': UNCTAD_IGO_list, 'NGO General Category accredited': UNCTAD_NGO_GEN_list, 'NGO Special Category accredited': UNCTAD_NGO_SPE_list},
        '60.0010': UNCTAD_merged_list,
        'default': ECOSOCList
    }

    # mapping used to determine automatically the representation type from an item of merged list
    # ex: if item is in UNCTAD_IGO_list, return 'Intergovernmental Organizations/IGO_accredited'
    _clientMapList = {
        '60.0010': {'Intergovernmental Organizations/IGO accredited': UNCTAD_IGO_list,
                    'Non Governmental Organizations/NGO General Category accredited': UNCTAD_NGO_GEN_list,
                    'Non Governmental Organizations/NGO Special Category accredited': UNCTAD_NGO_SPE_list,
                    'International Organizations of Parliamentarians/': UNCTAD_IOP_list
                    }
    }

    _contactMapping = {
        'Government': 'Delegate',
        'Intergovernmental Organizations': 'Observer Organization',
        'International Organizations of Parliamentarians': 'Observer Organization',
        'United Nations': 'UN Staff member',
        'Non Governmental Organizations': 'NGO',
        'Private Sector': 'Guest',
        'Academia': 'Guest',
        'Press/Media': 'Guest',
        'Other': 'Guest'
    }

    @classmethod
    def getRepTypeFromOrganization(self, client, orgName):
        if client in self._clientMapList:
            # search orgName in all lists of client
            for reptype in self._clientMapList[client]:
                if orgName in self._clientMapList[client][reptype]:
                    return reptype
        return None

    @classmethod
    def getUNSmapping(self, unsName):
        if unsName in UNS_mapping:
            return UNS_mapping[unsName]
        return None

    @classmethod
    def getContactTypeFromRep(cls, repType):
        try:
            return cls._contactMapping[repType]
        except KeyError:
            return 'Guest'

    @classmethod
    def getUNSystemName(cls, acronym, systemCat=None):
        # return the name associated with the acronym
        # acronym could be : "code" + param system,  or "code (system)"
        try:
            item = (acronym + " (" + systemCat + ")") if systemCat else acronym
            system = UNS_mapping[item]
            return system['name']
        except KeyError:
            return acronym

    @classmethod
    def getRepresentationTypes(cls, conf):
        # return Representation Type list for the matched client
        parent = conf.getOwner()
        while parent:
            if parent.getEmeetsClientCode() in cls.reprTypesMapping:
                return cls.reprTypesMapping[parent.getEmeetsClientCode()]
            parent = parent.getOwner()
        return cls.reprTypesMapping['default']
