# This file is part of Indico.
# Copyright (C) 2002 - 2015 European Organization for Nuclear Research (CERN).
#
# Indico is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# Indico is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Indico; if not, see <http://www.gnu.org/licenses/>.
import uuid

from MaKaC.common.cache import GenericCache
from MaKaC.common.mail import GenericMailer
from MaKaC.webinterface import urlHandlers
from MaKaC.common.info import HelperMaKaCInfo
from indico.core.config import Config
from MaKaC.i18n import _
from indico.core.logger import Logger

from indico.web.flask.util import url_for


def getSubjectIndicoTitle():
    minfo=HelperMaKaCInfo.getMaKaCInfoInstance()
    systitle="Indico"
    if minfo.getTitle().strip() != "":
        systitle=minfo.getTitle().strip()
    if minfo.getOrganisation().strip() != "":
        systitle="%s @ %s"%(systitle, minfo.getOrganisation().strip())
    return systitle

class personMail:

    def send(addto, addcc, addfrom, subject, body):
        addto = addto.replace("\r\n","")
        tolist = addto.split(",")
        cclist = addcc.split(",")
        maildata = {
            "fromAddr": addfrom,
            "toList": tolist,
            "ccList": cclist,
            "subject": subject,
            "body": body
            }
        GenericMailer.send(GenericNotification(maildata))
    send = staticmethod( send )


class GenericNotification :

    def __init__(self, data=None):
        if data is None:
            self._fromAddr = ""
            self._toList = []
            self._ccList = []
            self._bccList = []
            self._subject = ""
            self._body = ""
            self._contenttype = "text/html"
            self._attachments = []
        else:
            self._fromAddr = data.get("fromAddr", "")
            self._toList = data.get("toList", [])
            self._ccList = data.get("ccList", [])
            self._bccList = data.get("bccList", [])
            self._subject = data.get("subject", "")
            self._body = data.get("body", "")
            self._contenttype = data.get("content-type", "text/html")
            self._attachments = data.get("attachments", [])

    def getAttachments(self):
        return self._attachments

    def addAttachment(self, attachment):
        """
        Attachment is a dictionary with two keys:
        - name: containing the filename of the file
        - binary: containing the file data
        """

        self._attachments.append(attachment)

    def getContentType(self):
        return self._contenttype

    def setContentType(self, contentType):
        self._contenttype = contentType

    def getFromAddr(self):
        return self._fromAddr

    def setFromAddr(self, fromAddr):
        if fromAddr is None :
            return False
        self._fromAddr = fromAddr
        return True

    def getToList(self):
        return self._toList

    def setToList(self, toList):
        if toList is None :
            return False
        self._toList = toList
        return True

    def getCCList(self):
        return self._ccList

    def setCCList(self, ccList):
        if ccList is None :
            return False
        self._ccList = ccList
        return True

    def setBCCList(self, bccList):
        if bccList is None :
            return False
        self._bccList = bccList
        return True

    def getBCCList(self):
        return self._bccList

    def getSubject(self):
        return self._subject

    def setSubject(self, subject):
        if subject is None :
            return False
        self._subject = subject
        return True

    def getBody(self):
        return self._body

    def setBody(self, body):
        if body is None :
            return False
        self._body = body
        return True


class Mailer:

    def send( notification, fromAddress="" ):
        info = HelperMaKaCInfo.getMaKaCInfoInstance()
        if fromAddress.strip() == "":
            fromAddr = "%s <%s>"%(info.getTitle(), Config.getInstance().getSupportEmail())
        else:
            fromAddr = notification.getFromAddr()
        toAddr = str(notification.getDestination().getEmail())
        text = """%s
--
Indico project <http://indico-software.org/>
                """%(notification.getMsg())
        maildata = {
            "fromAddr": fromAddr,
            "toList": [toAddr],
            "subject": "[Indico] %s" % notification.getSubject(),
            "body": text
            }
        GenericMailer.send(GenericNotification(maildata))
    send = staticmethod( send )


def getEmailTemplate(name):
    from MaKaC.conference import CategoryManager
    template = filter(lambda k: k.getName() == name,CategoryManager().getDefaultConference().getEmailTemplateManager().getTemplates().values())
    return template[0] if len(template)>0 else None

def generic_send(user,templateName,event=None,to=None,sender=None):
    template = None

    if event:
        template = getEmailTemplate("{} - {}".format(templateName,event.getId()))

    if not template:
        template = getEmailTemplate(templateName)

    if not template:
        Logger.get("email.generic_send").warning("Template '{}' not found".format(templateName))
        return
    else:
        from MaKaC.webinterface.common import registrantNotificator
        template=template.getData()[1]
        d = {
            'to': to if to else [user.getEmail()],
            'from': sender if sender else Config.getInstance().getNoReplyEmail(),
            'subject': template['subject'],
            'body': template['body']
        }
        if event:
            d['conf']=event

        registrantNotificator.EmailNotificator().notify(user, d)


class sendConfirmationRequest:

    def __init__(self, user, conf=None):
        self._user = user
        self._conf = conf

    def send( self ):
        generic_send(self._user,"Account confirmation")


class sendAccountCreationModeration:

    def __init__( self, user ):
        self._user = user

    def send( self ):
        admins = [x.getEmail() for x in HelperMaKaCInfo.getMaKaCInfoInstance().getAdminList().getList()]
        generic_send(self._user,"Account moderation",to=admins)


class sendAccountCreationNotification:

    def __init__( self, user ):
        self._user = user

    def send( self ):
        admins = [x.getEmail() for x in HelperMaKaCInfo.getMaKaCInfoInstance().getAdminList().getList()]
        generic_send(self._user,"Account created",to=admins)


class sendAccountActivated:

    def __init__( self, user ):
        self._user = user

    def send( self ):
        generic_send(self._user,"Account enabled")


class sendAccountDisabled:

    def __init__( self, user ):
        self._user = user

    def send( self ):
        generic_send(self._user,"Account disabled")


class sendAccountLoginChanged:

    def __init__( self, user, oldEmail=None ):
        self._user = user
        self._oldEmail = oldEmail

    def send( self ):
        generic_send(self._user,"Account login changed", to=self._oldEmail)

def send_login_info(user, event=None):
    generic_send(user,"Reset password",event)
