# This file is part of Indico.
# Copyright (C) 2002 - 2015 European Organization for Nuclear Research (CERN).
#
# Indico is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# Indico is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Indico; if not, see <http://www.gnu.org/licenses/>.
from flask import request

import MaKaC.webinterface.pages.registrants as registrants
import MaKaC.webinterface.rh.conferenceDisplay as conferenceDisplay
import MaKaC.webinterface.common.regFilters as regFilters
from MaKaC.domain import DomainHolder
from MaKaC.errors import NoReportError, AccessError, MaKaCError, NotFoundError
import MaKaC.webinterface.displayMgr as displayMgr
from MaKaC.i18n import _
from indico.util.contextManager import ContextManager


class RHRegistrantsDisplayBase( conferenceDisplay.RHConferenceBaseDisplay):

    def _checkProtection( self ):
        conferenceDisplay.RHConferenceBaseDisplay._checkProtection(self)
        if not self._conf.getRegistrationForm().isActivated() or not self._conf.hasEnabledSection("regForm") or not displayMgr.ConfDisplayMgrRegistery().getDisplayMgr(self._conf).getMenu().getLinkByName("registrants").isEnabled():
            raise NoReportError("The registrants list page was disabled by the conference managers")

class RHShowRegistrant( RHRegistrantsDisplayBase):
    def _checkProtection( self ):
        ip = self.getAW().getIP()
        domains = DomainHolder().match({'name':'kiosk'})
        if len(domains)<1:
            raise NotImplementedError("Domain not found")
        else:
            domain = domains[0]
            belongsTo = domain.belongsTo(ip)

        if 'UNOG Registrant Kiosk' not in request.headers.get('User-Agent') or not belongsTo:
            raise AccessError("This feature is only available to UNOG Kiosks")

    def _checkParams(self, params):
        if params.get("confId")=='0' and params.get("registrantId")=='0':
            self._conf = None
            self._registrant = None
        else:
            RHRegistrantsDisplayBase._checkParams(self, params)
            regId = params.get("registrantId", None)
            if regId is None:
                raise MaKaCError(_("registrant id not set"))
            self._registrant = self._conf.getRegistrantById(regId)
            if self._registrant is None:
                raise NotFoundError(_("The registrant with id %s does not exist or has been deleted") % regId)

    def _process(self):
        p = registrants.WPRegistrantShow(self, self._conf, self._registrant)
        return p.display()

class RHRegistrantsList( RHRegistrantsDisplayBase ):

    @staticmethod
    def create_filter(conf, params, filterUsed=False, sessionFilterName="session"):
        regForm = conf.getRegistrationForm()
        newFilter = {}
        sessform =regForm.getSessionsForm()
        sesstypes = sessform.getSessionList()
        lsessions = []
        if not filterUsed:
            for sess in sesstypes:
                lsessions.append( sess.getId() )
        newFilter[sessionFilterName]=params.get("session",lsessions)
        return regFilters.RegFilterCrit(conf,newFilter)

    def _checkParams(self, params):
        RHRegistrantsDisplayBase._checkParams(self, params)

        # ---- FILTERING ----
        if params.has_key("firstChoice"):
            self._sessionFilterName="sessionfirstpriority"
        else:
            self._sessionFilterName="session"

        filterUsed=params.has_key("OK")
        self._filterCrit=self.create_filter(self._conf, params, filterUsed, self._sessionFilterName)

        sessionShowNoValue=True
        if filterUsed:
            sessionShowNoValue =  params.has_key("sessionShowNoValue")
        self._filterCrit.getField(self._sessionFilterName).setShowNoValue( sessionShowNoValue )

        # ---- SORTING ----
        self._sortingCrit = regFilters.SortingCriteria( [params.get( "sortBy", "Name" ).strip()] )
        self._order = params.get("order","down")

    def _process( self ):
        p = registrants.WPConfRegistrantsList( self, self._conf )
        return p.display(filterCrit=self._filterCrit, sortingCrit=self._sortingCrit, order=self._order, sessionFilterName=self._sessionFilterName)
