function resetInput() {
    var $el = $("div#fileUploadSection input[type='file']");
    $el.wrap('<form>').closest('form').get(0).reset();
    $el.unwrap();
}

function resizeHidden(scope,value) {
    if (value==null) {
        return;
    }

    var maxW = 300;
    var maxH = 300;
    if (value.lastIndexOf('data:',0)===0) {
        var img = new Image;
        img.onload = function () {
            var canvas = document.createElement("canvas");
            var ctx = canvas.getContext("2d");
            var iw = img.width;
            var ih = img.height;
            var scale = Math.min((maxW / iw), (maxH / ih));
            var iwScaled = iw * scale;
            var ihScaled = ih * scale;
            canvas.width = iwScaled;
            canvas.height = ihScaled;
            ctx.drawImage(img, 0, 0, iwScaled, ihScaled);
            var data = canvas.toDataURL("image/jpeg", 0.9);
            scope.field.pictureuri = data;
            resetInput();
            scope.$apply();
        };
        img.src = value;
    }
}

ndRegForm.directive('ndPictureField', function(url) {
    return {
        require: 'ndField',
        controller: function($scope) {
            $scope.tplInput = url.tpl('picture.tpl.html');
            if ($scope.userdata[$scope.fieldName]!=null)
                $scope.imagesrc = $scope.userdata[$scope.fieldName].path
            else
                $scope.imagesrc = "";
            $scope.field.pictureuri = '';

            $scope.$watch('imagesrc',function(newvalue,oldvalue) {
                resizeHidden($scope,newvalue);
            });
            $scope.fileInput = function(file) {
                $scope.imagesrc = 'data:'+file.mimetype+';'+file.encoding+','+file.content;
            }
        },

        link: function(scope,element) {
            scope.settings.fieldName = $T("Picture");
            scope.field.originalMandatory = scope.field.mandatory;
            scope.field.removeVisible=false;
            scope.removeAttachment = function() {
                delete scope.userdata[scope.getName(scope.field.input)];
            };
            scope.removeSnapshot = function() {
                $('#picture_preview').attr('src','');
                $('#picture_uri').val('');
                scope.field.removeVisible = false;
                scope.field.mandatory = scope.field.originalMandatory;
            };

            function customDarkroomDestroy(darkroom) {
                var parent = darkroom.containerElement.parentNode;
                var elem = document.getElementById('picture_preview');
                parent.appendChild(elem);
                while (parent.firstChild && parent.firstChild.id!='picture_preview') {
                    parent.removeChild(parent.firstChild);
                }
                elem.style.display='';
            }

            scope.enableDarkRoom = function() {
                scope.darkRoom = new Darkroom('#picture_preview',{
                    minWidth: 300,
                    minHeight: 300,
                    //maxWidth: 300,
                    //maxHeight: 300,
                    plugins: {
                        crop: {
                            ratio: 1
                        },
                        save: {
                            callback: function () {
                                customDarkroomDestroy(this.darkroom);
                                var newImage = scope.darkRoom.canvas.toDataURL("image/jpeg",0.9);
                                scope.imagesrc = newImage;
                                scope.$apply();
                            }
                        }
                    }
                });
            };
        }
    };
});

ndRegForm.value('Frozen',{frozen:false});

ndRegForm.controller('WebcamCtrl',function($scope,Frozen) {
    $scope.take_snapshot = function() {
        // take snapshot and get image data
        Webcam.snap(function (data_uri) {
            $scope.field.mandatory = false;
            $scope.field.removeVisible = true;
            $('#picture_preview').attr('src',data_uri);
            resizeHidden($scope,data_uri);
        });
        $('.qtip:visible').qtip("hide");
    };

    $scope.toggle_freeze = function() {
        if (Frozen.frozen) {
            Webcam.unfreeze();
            $('#freeze').text("Freeze");
            Frozen.frozen = false;
        } else {
            Webcam.freeze();
            $('#freeze').text("Unfreeze");
            Frozen.frozen = true;
        }
    };
});

ndRegForm.directive('ndWebcam', function ($http, $compile, $templateCache, url) {
    return {
        controller: 'WebcamCtrl',
        link: function (scope, element) {
            $http.get(url.tpl('webcam.tpl.html'), {cache: $templateCache})
                .success(function (template) {

                    var content = $compile(template)(scope);

                    element.qtip({
                        content: {
                            title: {
                                text: $T('Take picture')
                            },
                            text: content
                        },

                        position: {
                            my: 'top center',
                            at: 'bottom center'
                        },

                        show: {
                            event: 'click',
                            solo: true,
                            modal: {
                                on: true
                            }
                        },

                        hide: false,

                        style: {
                            classes: 'regform-camera-qtip',
                            name: 'light'
                        },

                        events: {
                            hide: function (event, api) {
                                Webcam.reset();
                            },
                            show: function (event, api) {
                                Webcam.set(
                                    {
                                        width: 480,
                                        height: 320,
                                        dest_width: 640,
                                        dest_height: 480,
                                        image_format: 'jpeg',
                                        jpeg_quality: 90
                                    }
                                );
                                var swfURL = url.tpl('webcam.swf');
                                Webcam.setSWFLocation(swfURL);
                                Webcam.attach('#my_camera');
                                var frozen = false;
                            }
                        }
                    });
                });
        }
    };
});
