from indico.core.config import Config
import cups
from flask import session


class PrintingSystem(object):
    __instance = None
    __conn = None
    __printers = None

    def __init__(self):
        self.initialize()

    @classmethod
    def getInstance(cls):
        """returns an instance of the PrintingSystem class ensuring only a single
           instance is created. All the clients should use this method for
           setting a PrintingSystem object instead of normal instantiation
        """
        if cls.__instance is None:
            cls.__instance = PrintingSystem()
        return cls.__instance

    def initialize(self):
        host = Config.getInstance().getPrintServerHost()
        if host:
            cups.setServer(host)

        port = Config.getInstance().getPrintServerPort()
        if port:
            cups.setPort(port)

        user = Config.getInstance().getPrintServerUser()
        if user:
            cups.setUser(user)

        password = Config.getInstance().getPrintServerPassword()
        if password:
            cups.setPasswordCB(lambda x:  password)

        try:
            self.__conn = cups.Connection()
        except Exception as e:
            cups.setServer('localhost')
            self.__conn = cups.Connection()

    def printers(self):
        if self.__printers is None:
            self.initialize()
            res = [str(i) for i in self.__conn.getPrinters()]
            self.__printers = res
        return self.__printers

    def refresh(self):
        self.printers()

    def printer(self):
        cupsDefault = self.__conn.getDefault()
        printer = session.get('printer')
        if printer is None:
            printer = session.user.getPrinter()
            if printer is None:
                printer = cupsDefault
            session.user.setPrinter(printer)
            session['printer']=printer
        return printer

    def setUserPrinter(self,user,printer):
        user.setPrinter(printer)
        session['printer'] = printer

    def printOnServer(self):
        printOnServer = session.get('printOnServer')
        if printOnServer is None:
            printOnServer = session.user.getPrintOnServer()
            if printOnServer is None:
                printOnServer = False
            session.user.setPrintOnServer(printOnServer)
            session['printOnServer'] = printOnServer
        return printOnServer

    def setUserPrintOnServer(self, user, printOnServer):
        user.setPrintOnServer(printOnServer)
        session['printOnServer'] = printOnServer

    def printJob(self,filename,printer=None):
        if not printer:
            printer = self.printer()
        self.__conn.printFile(printer,filename,'Badge',dict())